@@ -1,5 +1,5 @@
 /*
- *  $Id: master.h,v 5.15 2002-09-29 19:05:25-07 bryan Exp $
+ *  $Id: master.h,v 5.16 2003-03-06 10:13:41-08 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -37,10 +37,5 @@
 /*
  * stuff the master process needs
  */
-#if USE_ANSI_PROTO
-extern void Master(void);
-extern void SignalKids(int);
-#else
-extern void Master();
-extern void SignalKids();
-#endif
+extern void Master PARAMS((void));
+extern void SignalKids PARAMS((int));
